﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Drawing2D;
using System.IO;
using System.Drawing.Imaging;

namespace howto_images_on_polygon
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        private void btnGo_Click(object sender, EventArgs e)
        {
            try
            {
                Bitmap fg_image = new Bitmap(txtFgImage.Text);
                Bitmap bg_image = new Bitmap(txtBgImage.Text);
                int width = int.Parse(txtWidth.Text);
                int height = int.Parse(txtHeight.Text);
                int fg_width = int.Parse(txtFgWidth.Text);
                int radius = int.Parse(txtRadius.Text);
                int num_images = int.Parse(txtNumImages.Text);

                // Scale the foreground image.
                float scale = fg_width / (float)fg_image.Width;
                int fg_height = (int)(fg_image.Height * scale);
                fg_image = new Bitmap(fg_image, new Size(fg_width, fg_height));

                // Draw the final result.
                Bitmap bm = new Bitmap(width, height);
                using (Graphics gr = Graphics.FromImage(bm))
                {
                    gr.InterpolationMode = InterpolationMode.High;
                    gr.DrawImage(bg_image, 0, 0);
                    int cx = width / 2;
                    int cy = height / 2;
                    int rx = fg_width / 2;
                    int ry = fg_height / 2;
                    RectangleF src_rect =
                        new RectangleF(0, 0, fg_width, fg_height);

                    double theta = -Math.PI / 2;
                    double dtheta = 2 * Math.PI / num_images;
                    for (int i = 0; i < num_images; i++)
                    {
                        float x = (float)(radius * Math.Cos(theta));
                        float y = (float)(radius * Math.Sin(theta));
                        PointF[] dest_points =
                        {
                            new PointF(cx + x - rx, cy + y - ry),
                            new PointF(cx + x + rx, cy + y - ry),
                            new PointF(cx + x - rx, cy + y + ry),
                        };
                        gr.DrawImage(fg_image, dest_points, src_rect, GraphicsUnit.Pixel);
                        theta += dtheta;
                    }

                    // Redraw the left side of the first image.
                    src_rect =
                        new RectangleF(0, 0, fg_width / 2f, fg_height);

                    theta = -Math.PI / 2;
                    {
                        float x = (float)(radius * Math.Cos(theta));
                        float y = (float)(radius * Math.Sin(theta));
                        PointF[] dest_points =
                        {
                            new PointF(cx + x - rx, cy + y - ry),
                            new PointF(cx + x, cy + y - ry),
                            new PointF(cx + x - rx, cy + y + ry),
                        };
                        gr.DrawImage(fg_image, dest_points, src_rect, GraphicsUnit.Pixel);
                    }
                }
                picResult.Image = bm;
            }
            catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
        }

        private void mnuFileForegroundImage_Click(object sender, EventArgs e)
        {
            ofdImage.FileName = txtFgImage.Text;
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                txtFgImage.Text = ofdImage.FileName;
            }
        }

        private void mnuFileBackgroundImage_Click(object sender, EventArgs e)
        {
            ofdImage.FileName = txtBgImage.Text;
            if (ofdImage.ShowDialog() == DialogResult.OK)
            {
                txtBgImage.Text = ofdImage.FileName;
            }
        }

        private void mnuFileSaveAs_Click(object sender, EventArgs e)
        {
            if (sfdResult.ShowDialog() == DialogResult.OK)
            {
                SaveImage(picResult.Image, sfdResult.FileName);
            }
        }

        // Save the file with the appropriate format.
        public void SaveImage(Image image, string filename)
        {
            string extension = Path.GetExtension(filename);
            switch (extension.ToLower())
            {
                case ".bmp":
                    image.Save(filename, ImageFormat.Bmp);
                    break;
                case ".exif":
                    image.Save(filename, ImageFormat.Exif);
                    break;
                case ".gif":
                    image.Save(filename, ImageFormat.Gif);
                    break;
                case ".jpg":
                case ".jpeg":
                    image.Save(filename, ImageFormat.Jpeg);
                    break;
                case ".png":
                    image.Save(filename, ImageFormat.Png);
                    break;
                case ".tif":
                case ".tiff":
                    image.Save(filename, ImageFormat.Tiff);
                    break;
                default:
                    throw new NotSupportedException(
                        "Unknown file extension " + extension);
            }
        }

        private void mnuFileExit_Click(object sender, EventArgs e)
        {
            Close();
        }
    }
}
